if (sfeDoNotLoad) then
	return;
end

function firstCharIsNarrow(text)
	if (not text) then
		return false;
	end
	local firstChar = string.sub(text, 1, 1);
	return firstChar == "1"
		or levelTextFirstChar == "i"
		or levelTextFirstChar == "I"
		or levelTextFirstChar == "l"
		or levelTextFirstChar == ""
		or levelTextFirstChar == "!";
end

function SfeUpdateOrderHallMissionFrame()
	local list = OrderHallMissionFrame.MissionTab.MissionList;

	if (SFE_ORDERHALL_MISSIONS_ENHANCED) then
		list.listScroll:SetPoint("TOPLEFT", list, "TOPLEFT", 0, -6);
		list.listScroll:SetPoint("BOTTOMRIGHT", list, "BOTTOMRIGHT", -3, 7);
		list.listScroll.scrollBar:SetPoint("TOPLEFT", list.listScroll, "TOPRIGHT", -20, -18);
		list.listScroll.scrollBar:SetPoint("BOTTOMLEFT", list.listScroll, "BOTTOMRIGHT", -20, 13);
		list.listScroll.buttonHeight = 79 + (16.0 / #list.listScroll.buttons); -- Shame
		list.Tab1:SetFrameLevel(7);
		list.Border:Show();
	else
		list.listScroll:SetPoint("TOPLEFT", list, "TOPLEFT", 5, -6);
		list.listScroll:SetPoint("BOTTOMRIGHT", list, "BOTTOMRIGHT", -3, 8);
		list.listScroll.scrollBar:SetPoint("TOPLEFT", list.listScroll, "TOPRIGHT", -20, -12);
		list.listScroll.scrollBar:SetPoint("BOTTOMLEFT", list.listScroll, "BOTTOMRIGHT", -20, 12);
		list.listScroll.buttonHeight = 79;
		list.Tab1:SetFrameLevel(3);
		list.Border:Hide();
	end
end

function OrderHallMissionFrame_MissionTab_MissionList_Update_PostHook(self)
	if (not OrderHallMissionFrame or not self:IsShown()) then
		return;
	end

	local missions;
	if (self.showInProgress) then
		missions = self.inProgressMissions;
	else
		missions = self.availableMissions;
	end
	local numMissions = #missions;
	local scrollFrame = self.listScroll;
	local offset = HybridScrollFrame_GetOffset(scrollFrame);
	local buttons = scrollFrame.buttons;
	local numButtons = #buttons;

	if (SFE_ORDERHALL_MISSIONS_ENHANCED) then
		for i = 1, numButtons do
			local button = buttons[i];
			button:SetWidth(846);
			local index = offset + i;

			if (index <= numMissions) then
				local mission = missions[index];

				-- Time details
				if (not mission.inProgress) then
					local duration = mission.duration;
					if (mission.durationSeconds >= GARRISON_LONG_MISSION_TIME) then
						duration = string.format(GARRISON_LONG_MISSION_TIME_FORMAT, duration);
					end
					local durationText = string.format(PARENS_TEMPLATE, duration);
					local durationTextLength = string.len(durationText);
					if (string.sub(durationText, durationTextLength) == ")") then
						local expirationMessage = "";
						if (mission.offerEndTime) then
							local remainingSeconds = mission.offerEndTime - GetTime();
							local colorCode, colorCodeEnd = "", "";
							if (remainingSeconds < 8 * 3600) then
								colorCode, colorCodeEnd = RED_FONT_COLOR_CODE, FONT_COLOR_CODE_CLOSE;
							elseif (remainingSeconds < 24 * 3600) then
								colorCode, colorCodeEnd = YELLOW_FONT_COLOR_CODE, FONT_COLOR_CODE_CLOSE;
							end
							expirationMessage = ", "..colorCode..string.format(SFE_EXPIRES_IN, mission.offerTimeRemaining)..colorCodeEnd;
						end
						button.Summary:SetText(string.sub(durationText, 2, durationTextLength - 1)..expirationMessage);
					end
				end
				button.Title:SetPoint("LEFT", 165, 10);
				button.Title:SetWidth(655 - #mission.rewards * 65);
				button.Summary:ClearAllPoints();
				button.Summary:SetPoint("TOPLEFT", button.Title, "BOTTOMLEFT", 0, -4);

				-- Level
				local levelTextY = 0;
				if (mission.isMaxLevel and mission.iLevel > 0) then
					button.Level:SetText(mission.iLevel);
					button.ItemLevel:SetText(ITEM_LEVEL_ABBR);
					levelTextY = - button.ItemLevel:GetHeight() / 2;
				end
				button.Level:SetPoint("CENTER", button, "LEFT", 40, levelTextY);
				-- WoW calculates fonts as monospaced, which visually is false, since '1' is thin it doesn't look centered
				local iLevelTextX = 0;
				if (firstCharIsNarrow(button.Level:GetText())) then
					iLevelTextX = iLevelTextX + 3;
				end
				if (firstCharIsNarrow(button.ItemLevel:GetText())) then
					iLevelTextX = iLevelTextX - 2;
				end
				button.ItemLevel:ClearAllPoints();
				button.ItemLevel:SetPoint("BOTTOM", button.Level, "TOP", iLevelTextX, 2);

				-- Rarity
				if (mission.isRare) then
					button.Level:SetTextColor(0.098, 0.537, 0.969, 1.0);
					button.ItemLevel:SetTextColor(0.098, 0.537, 0.969, 1.0);
				else
					button.Level:SetTextColor(0.84, 0.72, 0.57, 1.0);
					button.ItemLevel:SetTextColor(0.84, 0.72, 0.57, 1.0);
				end
				button.RareText:Hide();

				-- Rewards
				local allMissionRewards = {};
				local allMissionRewardsCount = 0;
				if (mission.rewards) then
					for i = 1, #mission.rewards do
						allMissionRewardsCount = allMissionRewardsCount + 1;
						allMissionRewards[allMissionRewardsCount] = mission.rewards[i];
					end
				end
				if (mission.overmaxRewards) then
					for i = 1, #mission.overmaxRewards do
						allMissionRewardsCount = allMissionRewardsCount + 1;
						allMissionRewards[allMissionRewardsCount] = mission.overmaxRewards[i];
					end
				end
				if (button.Title:GetWidth() > 655 - 12 - allMissionRewardsCount * 56) then
					button.Title:SetWidth(655 - 12 - allMissionRewardsCount * 56);
				end
				if (button.Summary:GetWidth() > 655 - 12 - allMissionRewardsCount * 56) then
					button.Summary:SetWidth(655 - 12 - allMissionRewardsCount * 56);
				end
				GarrisonMissionButton_SetRewards(button, allMissionRewards);
				for i = #button.Rewards, 1, -1 do
					button.Rewards[i]:SetSize(48, 48);
					if (i <= allMissionRewardsCount) then
						button.Rewards[i]:Show();
					else
						button.Rewards[i]:Hide();
					end
					-- Move to function
					if (not button.Rewards[i].originalRightPoint) then
						for j = 1, button.Rewards[i]:GetNumPoints() do
							local point, relativeTo, relativePoint, xOffset, yOffset = button.Rewards[i]:GetPoint(j);
							if (point == "RIGHT") then
								button.Rewards[i].originalRightPoint = { relativeTo, relativePoint, xOffset, yOffset, SFE_ORDERHALL_MISSIONS_ENHANCED };
							end
						end
					end
					if (i ~= 1) then
						button.Rewards[i]:SetPoint("RIGHT", button.Rewards[i - 1], "LEFT", 8 + 48 + 48, 0);
					else
						if (gmmRunning and not self.showInProgress) then
							if (button.Rewards[i].gmmChangedX) then
								button.Rewards[i]:SetPoint("RIGHT", button, "RIGHT", -72 - 56 * (allMissionRewardsCount - 1), 0);
							else
								local _, _, _, oldX = button.Rewards[i]:GetPoint("RIGHT");
								local newX = -12 - 56 * (allMissionRewardsCount - 1);
								if (button.Rewards[i].oldX and button.Rewards[i].oldX - oldX > 59) then
									button.Rewards[i].gmmChangedX = true;
									newX = newX - 60;
								end
								button.Rewards[i]:SetPoint("RIGHT", button, "RIGHT", newX, 0);
								if (button.Rewards[i].gmmChangedX) then
									button.Rewards[i].oldX = nil;
								else
									button.Rewards[i].oldX = newX;
								end
							end
						else
							button.Rewards[i]:SetPoint("RIGHT", button, "RIGHT", -12 - 56 * (allMissionRewardsCount - 1), 0);
						end
					end
					button.Rewards[i].Icon:SetMask("Interface\\ICONS\\INV_Helmet_47");
					if (not button.Rewards[i].bonusIcon) then
						button.Rewards[i].bonusIcon = CreateFrame("Frame", nil, button);
						button.Rewards[i].bonusIcon:SetSize(28, 28);
						button.Rewards[i].bonusIcon:SetPoint("CENTER", button.Rewards[i], "TOPRIGHT", -8, -8);
						button.Rewards[i].bonusIcon:SetHitRectInsets(4, 4, 1, 4);
						button.Rewards[i].bonusIcon.texture = button.Rewards[i].bonusIcon:CreateTexture(nil, "ARTWORK", nil, 2);
						button.Rewards[i].bonusIcon.texture:SetAtlas("honorsystem-icon-bonus");
						button.Rewards[i].bonusIcon.texture:SetSize(28, 28);
						button.Rewards[i].bonusIcon.texture:SetPoint("CENTER", button.Rewards[i].bonusIcon, "CENTER", 0, 0);
						button.Rewards[i].bonusIcon:SetScript("OnEnter", function(self, motion)
								GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
								GameTooltip:SetText(BONUS_REWARDS, nil, nil, nil, nil, true);
							end);
						button.Rewards[i].bonusIcon:SetScript("OnLeave", GameTooltip_Hide);
					end
					if (i > #mission.rewards) and (i <= allMissionRewardsCount) then
						button.Rewards[i].bonusIcon:Show();
					else
						button.Rewards[i].bonusIcon:Hide();
					end
				end
			end
		end
	else
		for i = 1, numButtons do
			local button = buttons[i];
			button:SetWidth(832);
			local index = offset + i;

			if (index <= numMissions) then
				local mission = missions[index];

				-- Level
				if (mission.isRare and mission.isMaxLevel and mission.iLevel > 0) then
					button.Level:SetPoint("CENTER", button, "TOPLEFT", 40, -22);
				else
					button.Level:SetPoint("CENTER", button, "TOPLEFT", 40, -36);
				end
				if (mission.isRare) then
					button.RareText:Show();
				end
				button.ItemLevel:ClearAllPoints();
				button.ItemLevel:SetPoint("TOP", button.Level, "BOTTOM", 0, 2);

				-- Rarity
				button.Level:SetTextColor(0.84, 0.72, 0.57, 1.0);
				button.ItemLevel:SetTextColor(0.84, 0.72, 0.57, 1.0);

				-- Rewards
				for i = 1, #button.Rewards do
					if (not button.Rewards[i].originalRightPoint) then
						for j = 1, button.Rewards[i]:GetNumPoints() do
							local point, relativeTo, relativePoint, xOffset, yOffset = button.Rewards[i]:GetPoint(j);
							if (point == "RIGHT") then
								button.Rewards[i].originalRightPoint = { relativeTo, relativePoint, xOffset, yOffset, SFE_ORDERHALL_MISSIONS_ENHANCED };
							end
						end
					end
					button.Rewards[i]:SetSize(72, 72);
					if (button.Rewards[i].originalRightPoint) then
						button.Rewards[i]:SetPoint("RIGHT", button.Rewards[i].originalRightPoint[1], button.Rewards[i].originalRightPoint[2], button.Rewards[i].originalRightPoint[3], button.Rewards[i].originalRightPoint[4]);
						if (gmmRunning and not button.Rewards[i].originalRightPoint[5]) then
							button.Rewards[i]:SetPoint("RIGHT", button.Rewards[i].originalRightPoint[1], button.Rewards[i].originalRightPoint[2], button.Rewards[i].originalRightPoint[3] - 60, button.Rewards[i].originalRightPoint[4]);
						end
					end
					button.Rewards[i].Icon:SetMask(nil);
					if (button.Rewards[i].bonusIcon) then
						button.Rewards[i].bonusIcon:Hide();
					end
				end
			end
		end
	end
			print("sfe>");
end

--function GarrisonMissionButton_OnEnter_Enhanced(self, button, oldFunction)
--	oldFunction(self, button);
--
--	if (self.info == nil) then
--		oldFunction(self, button);
--		return;
--	end
--end

local SfeLoadController = CreateFrame("Frame");
SfeLoadController:RegisterEvent("ADDON_LOADED");
SfeLoadController:SetScript("OnEvent", function(self, event, name)
		if ((event == "ADDON_LOADED") and (name == "StandardFramesEnhanced_OrderHall")) then
			if (SFE_ORDERHALL_MISSIONS_ENHANCED == nil) then
				SFE_ORDERHALL_MISSIONS_ENHANCED = true;
			end

			if (OrderHallMissionFrame.EnhancedCheckbox == nil) then
				OrderHallMissionFrame.EnhancedCheckbox = CreateFrame("CheckButton", "OrderHallMissionFrameEnhancedCheckbox", OrderHallMissionFrame, "OptionsBaseCheckButtonTemplate");
				OrderHallMissionFrame.EnhancedCheckbox:SetHitRectInsets(0, 0, 0, 0);

				OrderHallMissionFrame.EnhancedCheckbox:SetPoint("RIGHT", OrderHallMissionFrame.CloseButton, "LEFT", 7, 0);
				OrderHallMissionFrame.EnhancedCheckbox:SetAlpha(0.1);
				OrderHallMissionFrame.EnhancedCheckbox:SetScript("OnClick", function(self, button, down)
						SFE_ORDERHALL_MISSIONS_ENHANCED = self:GetChecked();
						SfeUpdateOrderHallMissionFrame();
						OrderHallMissionFrame.MissionTab.MissionList:Update(OrderHallMissionFrame.MissionTab.MissionList);
						if (SFE_ORDERHALL_MISSIONS_ENHANCED) then
							PlaySound("igMainMenuOptionCheckBoxOn");
						else
							PlaySound("igMainMenuOptionCheckBoxOff");
						end
					end);
				OrderHallMissionFrame.EnhancedCheckbox:SetScript("OnEnter", function(self, motion)
						self:SetAlpha(1.0);
						GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
						GameTooltip:SetText(SFE_ENHANCED_UI, nil, nil, nil, nil, true);
					end);
				OrderHallMissionFrame.EnhancedCheckbox:SetScript("OnLeave", function(self, motion)
						self:SetAlpha(0.1);
						GameTooltip_Hide(self, motion);
					end);
			end
			OrderHallMissionFrame.EnhancedCheckbox:SetChecked(SFE_ORDERHALL_MISSIONS_ENHANCED);

			OrderHallMissionFrame.MissionTab.MissionList.Border = CreateFrame("Frame", nil, OrderHallMissionFrame.MissionTab.MissionList.listScroll);
			OrderHallMissionFrame.MissionTab.MissionList.Border:SetPoint("TOPLEFT", OrderHallMissionFrame.MissionTab.MissionList.listScroll, "TOPLEFT", 0, 6);
			OrderHallMissionFrame.MissionTab.MissionList.Border:SetPoint("BOTTOMRIGHT", OrderHallMissionFrame.MissionTab.MissionList.listScroll, "BOTTOMRIGHT", 3, -7);
			local t;
			t = OrderHallMissionFrame.MissionTab.MissionList.Border:CreateTexture(nil, "BORDER", 4);
			t:SetAtlas("_Garr_InfoBoxBorderMission-Top", true);
			t:SetHorizTile(true);
			t:SetPoint("TOPLEFT");
			t:SetPoint("TOPRIGHT");
			t = OrderHallMissionFrame.MissionTab.MissionList.Border:CreateTexture(nil, "BORDER", 4);
			t:SetAtlas("_Garr_InfoBoxBorderMission-Top", true);
			t:SetHorizTile(true);
			t:SetTexCoord(0, 1, 1, 0);
			t:SetPoint("BOTTOMLEFT");
			t:SetPoint("BOTTOMRIGHT");
			t = OrderHallMissionFrame.MissionTab.MissionList.Border:CreateTexture(nil, "BORDER", 4);
			t:SetAtlas("!Garr_InfoBoxBorderMission-Left", true);
			t:SetVertTile(true);
			t:SetPoint("TOPLEFT");
			t:SetPoint("BOTTOMLEFT");
			t = OrderHallMissionFrame.MissionTab.MissionList.Border:CreateTexture(nil, "BORDER", 4);
			t:SetAtlas("!Garr_InfoBoxBorderMission-Left", true);
			t:SetVertTile(true);
			t:SetTexCoord(1, 0, 0, 1);
			t:SetPoint("TOPRIGHT");
			t:SetPoint("BOTTOMRIGHT");
			t = OrderHallMissionFrame.MissionTab.MissionList.Border:CreateTexture(nil, "BORDER", 5);
			t:SetAtlas("Garr_InfoBoxBorderMission-Corner", true);
			t:SetPoint("TOPLEFT");
			t = OrderHallMissionFrame.MissionTab.MissionList.Border:CreateTexture(nil, "BORDER", 5);
			t:SetAtlas("Garr_InfoBoxBorderMission-Corner", true);
			t:SetTexCoord(1, 0, 0, 1);
			t:SetPoint("TOPRIGHT");
			t = OrderHallMissionFrame.MissionTab.MissionList.Border:CreateTexture(nil, "BORDER", 5);
			t:SetAtlas("Garr_InfoBoxBorderMission-Corner", true);
			t:SetTexCoord(0, 1, 1, 0);
			t:SetPoint("BOTTOMLEFT");
			t = OrderHallMissionFrame.MissionTab.MissionList.Border:CreateTexture(nil, "BORDER", 5);
			t:SetAtlas("Garr_InfoBoxBorderMission-Corner", true);
			t:SetTexCoord(1, 0, 1, 0);
			t:SetPoint("BOTTOMRIGHT");
			OrderHallMissionFrame.MissionTab.MissionList.Border:Hide();

			SfeUpdateOrderHallMissionFrame();

			local oldOrderHallMissionFrame_MissionTab_MissionList_Update = OrderHallMissionFrame.MissionTab.MissionList.Update;
			function OrderHallMissionFrame.MissionTab.MissionList:Update()
				oldOrderHallMissionFrame_MissionTab_MissionList_Update(self);
				OrderHallMissionFrame_MissionTab_MissionList_Update_PostHook(self);
			end
			--local oldGarrisonMissionButton_OnEnter = GarrisonMissionButton_OnEnter;
			--function GarrisonMissionButton_OnEnter(self, button)
			--	GarrisonMissionButton_OnEnter_Enhanced(self, button, oldGarrisonMissionButton_OnEnter);
			--end

			if (gmmRunning) then
				SfeLoadController:RegisterEvent("GARRISON_MISSION_NPC_OPENED");
		f = function(self)
					local buttons = OrderHallMissionFrame.MissionTab.MissionList.listScroll.buttons;
					if (SFE_ORDERHALL_MISSIONS_ENHANCED) then
						for i = 1, #buttons do
							buttons[i].Level:SetTextColor(1, 0, 0, 1.0);
							buttons[i].Level:SetText("AAA");
						end
					end
				end
			local oldA = OrderHallMissionFrame.MissionTab.MissionList.Update;
			OrderHallMissionFrame.MissionTab.MissionList.Update = function(self)
					oldA(self);
					f(self);
				end;
--			print(">SFE>");
--			hooksecurefunc(OrderHallMissionFrame.MissionTab.MissionList, "Update", f);
--			hooksecurefunc(OrderHallMissionFrame.MissionTab.MissionList.listScroll, "update", f);
--			print("SFE>");
			end
		elseif (gmmRunning and event == "GARRISON_MISSION_NPC_OPENED") then
			SfeLoadController:UnregisterEvent("GARRISON_MISSION_NPC_OPENED");
		end
	end);
